package mudu.tv.mudutv_android_player;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.graphics.Color;
import android.media.AudioManager;
import android.util.AttributeSet;
import android.view.GestureDetector;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.MediaController;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.mudu.yaguplayer.video.widget.media.IMediaController;
import com.mudu.yaguplayer.video.widget.media.MuduVideoView;

/**
 * 自定义控制栏
 */
public class CustomController extends RelativeLayout implements IMediaController,
        View.OnTouchListener, View.OnClickListener, SeekBar.OnSeekBarChangeListener {

    public static final int FULLSCREEN_ID = R.id.full_id;

    private LinearLayout llBottom;
    private ImageView btnStart;
    private TextView tvCurrent;
    private TextView tvTotal;
    private SeekBar seekBar;
    private ImageView ivFullscreen;

    private Activity mActivity;
    private MuduVideoView mPlayer;
    private boolean mShowing;
    private boolean mPostProgress = false;
    private boolean mPostDismiss = false;
    //触摸显示后隐藏的时间
    private final int mDismissControlTime = 2500;
    //seek touch
    private boolean mHadSeekTouch = false;
    //当前是否全屏
    private boolean mIfCurrentIsFullscreen = false;
    //原始父view
    private ViewGroup mRawViewGroup;

    private static final int LAND_TYPE_NULL = 0;
    private static final int LAND_TYPE_NORMAL = 1;
    private static final int LAND_TYPE_REVERSE = 2;
    private int mIsLand = LAND_TYPE_NULL;

    //手势控制
    //触摸的X
    protected float mDownX;
    //触摸的Y
    protected float mDownY;
    //移动的Y
    protected float mMoveY;
    //是否改变音量
    protected boolean mChangeVolume = false;
    //是否改变播放进度
    protected boolean mChangePosition = false;
    //是否改变亮度
    protected boolean mBrightness = false;
    //是否首次触摸
    protected boolean mFirstTouch = false;
    //手势偏差值
    protected int mThreshold = 80;
    //手动滑动的起始偏移位置
    protected int mSeekEndOffset;
    //手指放下的位置
    protected int mDownPosition;
    //手势调节音量的大小
    protected int mGestureDownVolume;
    //音频焦点的监听
    protected AudioManager mAudioManager;
    //手动改变滑动的位置
    protected int mSeekTimePosition;
    //触摸滑动进度的比例系数
    protected float mSeekRatio = 1;
    //亮度
    protected float mBrightnessData = -1;

    protected Dialog mProgressDialog;
    protected ProgressBar mDialogProgressBar;
    protected TextView mDialogSeekTime;
    protected TextView mDialogTotalTime;
    protected Dialog mVolumeDialog;
    protected ProgressBar mDialogVolumeProgressBar;
    protected Dialog mBrightnessDialog;
    protected TextView mBrightnessDialogTv;

    public CustomController(@NonNull Context context) {
        this(context, null);
    }

    public CustomController(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CustomController(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        mActivity = (Activity) context;
        View view = inflate(context, R.layout.video_layout_normal, this);
        initView(view);
    }

    @SuppressLint("ClickableViewAccessibility")
    private void initView(View view) {
        btnStart = findViewById(R.id.iv_start);
        llBottom = findViewById(R.id.ll_bottom);
        tvCurrent = findViewById(R.id.tv_current);
        tvTotal = findViewById(R.id.tv_total);
        seekBar = findViewById(R.id.progress);
        ivFullscreen = findViewById(R.id.iv_fullscreen);

        btnStart.setOnTouchListener(this);
        btnStart.setOnClickListener(this);
        seekBar.setOnTouchListener(this);
        seekBar.setOnSeekBarChangeListener(this);
        ivFullscreen.setOnClickListener(this);
        ivFullscreen.setOnTouchListener(this);
        setOnClickListener(this);
        setOnTouchListener(this);

        mSeekEndOffset = CommonUtil.dip2px(getContext(), 50);
        mAudioManager = (AudioManager) getContext().getApplicationContext().getSystemService(Context.AUDIO_SERVICE);
    }

    private void updateStartPlay() {
        if (mPlayer.isPlaying()) {
            btnStart.setImageResource(R.drawable.video_click_pause_selector);
        } else {
            btnStart.setImageResource(R.drawable.video_click_play_selector);
        }
    }

    @Override
    public void hide() {
        hideAllWidget();
    }

    @Override
    public boolean isShowing() {
        return mShowing;
    }

    @Override
    public void setAnchorView(View view) {
    }

    @Override
    public void setEnabled(boolean enabled) {
    }

    @Override
    public void setMediaPlayer(MediaController.MediaPlayerControl player) {
        mPlayer = (MuduVideoView) player;
        updateStartPlay();
    }

    @Override
    public void show(int timeout) {
        showAllWidget();
        startDismissControlViewTimer();
    }

    @Override
    public void show() {
        show(mDismissControlTime);
    }

    @Override
    public void onClick(View v) {
        if (v.getId() == R.id.iv_start) {
            if (mPlayer == null) {
                return;
            }
            if (mPlayer.isError()) {
                mPlayer.start();
                updateStartPlay();
                return;
            }

            if (mPlayer.isPlaying()) {
                mPlayer.pause();
            } else {
                mPlayer.start();
            }
            updateStartPlay();
        } else if (v.getId() == R.id.iv_fullscreen) {
            if (mIfCurrentIsFullscreen) {
                clearFullscreenLayout();
            } else {
                startWindowFullscreen();
            }
        }
    }

    private final GestureDetector gestureDetector = new GestureDetector(getContext().getApplicationContext(),
            new GestureDetector.SimpleOnGestureListener() {
                @Override
                public boolean onSingleTapConfirmed(MotionEvent e) {
                    if (!mChangePosition && !mChangeVolume && !mBrightness) {
                        if (mPlayer != null && mPlayer.isInPlaybackState()) {
                            if (mShowing) {
                                hideAllWidget();
                            } else {
                                showAllWidget();
                            }
                        }
                    }
                    return super.onSingleTapConfirmed(e);
                }

            });

    @Override
    public boolean onTouch(View v, MotionEvent event) {
        if (v.getId() == R.id.progress) {
            switch (event.getAction()) {
                case MotionEvent.ACTION_DOWN:
                    cancelDismissControlViewTimer();
                case MotionEvent.ACTION_MOVE:
                    cancelProgressTimer();
                    break;
                case MotionEvent.ACTION_UP:
                    startProgressTimer();
                    startDismissControlViewTimer();
                    break;
            }
        } else if (v.getId() == R.id.iv_start) {
            switch (event.getAction()) {
                case MotionEvent.ACTION_DOWN:
                    cancelDismissControlViewTimer();
                    break;
                case MotionEvent.ACTION_UP:
                    startDismissControlViewTimer();
                    break;
            }
        }

        if (v instanceof CustomController) {
            float x = event.getX();
            float y = event.getY();
            switch (event.getAction()) {
                case MotionEvent.ACTION_DOWN:
                    touchControllerDown(x, y);
                    break;
                case MotionEvent.ACTION_MOVE:
                    float deltaX = x - mDownX;
                    float deltaY = y - mDownY;
                    float absDeltaX = Math.abs(deltaX);
                    float absDeltaY = Math.abs(deltaY);

                    if (!mChangePosition && !mChangeVolume && !mBrightness) {
                        touchControllerMoveFullLogic(absDeltaX, absDeltaY);
                    }
                    touchControllerMove(deltaX, deltaY, y);
                    break;
                case MotionEvent.ACTION_UP:
                    startDismissControlViewTimer();
                    touchControllerUp();
                    startProgressTimer();
                    break;
            }
            gestureDetector.onTouchEvent(event);
        }

        return false;
    }

    private void touchControllerDown(float x, float y) {
        mDownX = x;
        mDownY = y;
        mMoveY = 0;
        mChangeVolume = false;
        mChangePosition = false;
        mBrightness = false;
        mFirstTouch = true;
    }

    private void touchControllerMoveFullLogic(float absDeltaX, float absDeltaY) {
        int curWidth = CommonUtil.getCurrentScreenLand((Activity) getContext()) ?
                CommonUtil.getScreenHeight(getContext()) : CommonUtil.getScreenWidth(getContext());
        if (absDeltaX > mThreshold || absDeltaY > mThreshold) {
            cancelProgressTimer();
            if (absDeltaX >= mThreshold) {
                if (!mPlayer.isPlaying() && !mPlayer.isPaused()) {
                    //不是播放、暂停状态 不使用
                    return;
                }
                //防止全屏虚拟按键
                int screenWidth = CommonUtil.getScreenWidth(getContext());
                if (Math.abs(screenWidth - mDownX) > mSeekEndOffset) {
                    mChangePosition = true;
                    mDownPosition = getCurrentPositionWhenPlaying();
                }
            } else {
                int screenHeight = CommonUtil.getScreenHeight(getContext());
                boolean noEnd = Math.abs(screenHeight - mDownY) > mSeekEndOffset;
                if (mFirstTouch) {
                    mBrightness = (mDownX < curWidth * 0.5f) && noEnd;
                    mFirstTouch = false;
                }
                if (!mBrightness) {
                    mChangeVolume = noEnd;
                    mGestureDownVolume = mAudioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
                }
            }
        }
    }

    private void touchControllerMove(float deltaX, float deltaY, float y) {
        int screenWidth = CommonUtil.getScreenWidth(getContext());
        int screenHeight = CommonUtil.getScreenHeight(getContext());

        int curWidth = CommonUtil.getCurrentScreenLand((Activity) getContext()) ? screenHeight : screenWidth;
        int curHeight = CommonUtil.getCurrentScreenLand((Activity) getContext()) ? screenWidth : screenHeight;

        if (mChangePosition) {
            int totalTimeDuration = getDuration();
            mSeekTimePosition = (int) (mDownPosition + (deltaX * totalTimeDuration / curWidth) / mSeekRatio);
            if (mSeekTimePosition > totalTimeDuration)
                mSeekTimePosition = totalTimeDuration;
            String seekTime = CommonUtil.stringForTime(mSeekTimePosition);
            String totalTime = CommonUtil.stringForTime(totalTimeDuration);
            showProgressDialog(deltaX, seekTime, mSeekTimePosition, totalTime, totalTimeDuration);
            hideAllWidget();
        } else if (mChangeVolume) {
            deltaY = -deltaY;
            int max = mAudioManager.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
            int deltaV = (int) (max * deltaY * 3 / curHeight);
            mAudioManager.setStreamVolume(AudioManager.STREAM_MUSIC, mGestureDownVolume + deltaV, 0);
            int volumePercent = (int) (mGestureDownVolume * 100 / max + deltaY * 3 * 100 / curHeight);
            showVolumeDialog(-deltaY, volumePercent);
            hideAllWidget();
        } else if (mBrightness) {
            if (Math.abs(deltaY) > mThreshold) {
                float percent = (-deltaY / curHeight);
                onBrightnessSlide(percent);
                mDownY = y;
            }
            hideAllWidget();
        }
    }

    /**
     * 滑动改变亮度
     */
    private void onBrightnessSlide(float percent) {
        mBrightnessData = ((Activity) getContext()).getWindow().getAttributes().screenBrightness;
        if (mBrightnessData <= 0.00f) {
            mBrightnessData = 0.50f;
        } else if (mBrightnessData < 0.01f) {
            mBrightnessData = 0.01f;
        }
        WindowManager.LayoutParams lpa = ((Activity) getContext()).getWindow().getAttributes();
        lpa.screenBrightness = mBrightnessData + percent;
        if (lpa.screenBrightness > 1.0f) {
            lpa.screenBrightness = 1.0f;
        } else if (lpa.screenBrightness < 0.01f) {
            lpa.screenBrightness = 0.01f;
        }
        showBrightnessDialog(lpa.screenBrightness);
        ((Activity) getContext()).getWindow().setAttributes(lpa);
    }

    private void touchControllerUp() {
        dismissProgressDialog();
        dismissVolumeDialog();
        dismissBrightnessDialog();
        if (mChangePosition && mPlayer != null && (mPlayer.isPlaying() || mPlayer.isPaused())) {
            try {
                mPlayer.seekTo(mSeekTimePosition);
            } catch (Exception e) {
                e.printStackTrace();
            }
            int duration = getDuration();
            int progress = mSeekTimePosition * 100 / (duration == 0 ? 1 : duration);
            if (seekBar != null) {
                seekBar.setProgress(progress);
            }
            showAllWidget();
        }
    }

    /**
     * 快进、快退 Dialog
     */
    private void showProgressDialog(float deltaX, String seekTime, int seekTimePosition, String totalTime, int totalTimeDuration) {
        if (mProgressDialog == null) {
            View localView = LayoutInflater.from(getContext()).inflate(R.layout.video_progress_dialog, null);
            if (localView.findViewById(R.id.duration_progressbar) instanceof ProgressBar) {
                mDialogProgressBar = ((ProgressBar) localView.findViewById(R.id.duration_progressbar));
            }
            if (localView.findViewById(R.id.tv_current) instanceof TextView) {
                mDialogSeekTime = ((TextView) localView.findViewById(R.id.tv_current));
            }
            if (localView.findViewById(R.id.tv_duration) instanceof TextView) {
                mDialogTotalTime = ((TextView) localView.findViewById(R.id.tv_duration));
            }
            mProgressDialog = new Dialog(getContext(), R.style.video_style_dialog_progress);
            mProgressDialog.setContentView(localView);
            mProgressDialog.getWindow().addFlags(Window.FEATURE_ACTION_BAR);
            mProgressDialog.getWindow().addFlags(32);
            mProgressDialog.getWindow().addFlags(16);
            mProgressDialog.getWindow().setLayout(getWidth(), getHeight());
            WindowManager.LayoutParams localLayoutParams = mProgressDialog.getWindow().getAttributes();
            localLayoutParams.gravity = Gravity.TOP;
            localLayoutParams.width = getWidth();
            localLayoutParams.height = getHeight();
            int location[] = new int[2];
            getLocationOnScreen(location);
            localLayoutParams.x = location[0];
            localLayoutParams.y = location[1];
            mProgressDialog.getWindow().setAttributes(localLayoutParams);
        }
        if (!mProgressDialog.isShowing()) {
            mProgressDialog.show();
        }
        if (mDialogSeekTime != null) {
            mDialogSeekTime.setText(seekTime);
        }
        if (mDialogTotalTime != null) {
            mDialogTotalTime.setText(" / " + totalTime);
        }
        if (totalTimeDuration > 0)
            if (mDialogProgressBar != null) {
                mDialogProgressBar.setProgress(seekTimePosition * 100 / totalTimeDuration);
            }
    }

    /**
     * 音量 Dialog
     */
    private void showVolumeDialog(float deltaY, int volumePercent) {
        if (mVolumeDialog == null) {
            View localView = LayoutInflater.from(getContext()).inflate(R.layout.video_volume_dialog, null);
            if (localView.findViewById(R.id.volume_progressbar) instanceof ProgressBar) {
                mDialogVolumeProgressBar = ((ProgressBar) localView.findViewById(R.id.volume_progressbar));
            }
            mVolumeDialog = new Dialog(getContext(), R.style.video_style_dialog_progress);
            mVolumeDialog.setContentView(localView);
            mVolumeDialog.getWindow().addFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE);
            mVolumeDialog.getWindow().addFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL);
            mVolumeDialog.getWindow().addFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            mVolumeDialog.getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
            WindowManager.LayoutParams localLayoutParams = mVolumeDialog.getWindow().getAttributes();
            localLayoutParams.gravity = Gravity.TOP | Gravity.START;
            localLayoutParams.width = getWidth();
            localLayoutParams.height = getHeight();
            int location[] = new int[2];
            getLocationOnScreen(location);
            localLayoutParams.x = location[0];
            localLayoutParams.y = location[1];
            mVolumeDialog.getWindow().setAttributes(localLayoutParams);
        }
        if (!mVolumeDialog.isShowing()) {
            mVolumeDialog.show();
        }
        if (mDialogVolumeProgressBar != null) {
            mDialogVolumeProgressBar.setProgress(volumePercent);
        }
    }

    /**
     * 触摸亮度 Dialog
     */
    private void showBrightnessDialog(float percent) {
        if (mBrightnessDialog == null) {
            View localView = LayoutInflater.from(getContext()).inflate(R.layout.video_brightness, null);
            if (localView.findViewById(R.id.app_video_brightness) instanceof TextView) {
                mBrightnessDialogTv = (TextView) localView.findViewById(R.id.app_video_brightness);
            }
            mBrightnessDialog = new Dialog(getContext(), R.style.video_style_dialog_progress);
            mBrightnessDialog.setContentView(localView);
            mBrightnessDialog.getWindow().addFlags(WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE);
            mBrightnessDialog.getWindow().addFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCH_MODAL);
            mBrightnessDialog.getWindow().addFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);
            mBrightnessDialog.getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_HIDE_NAVIGATION);
            mBrightnessDialog.getWindow().setLayout(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
            WindowManager.LayoutParams localLayoutParams = mBrightnessDialog.getWindow().getAttributes();
            localLayoutParams.gravity = Gravity.TOP | Gravity.END;
            localLayoutParams.width = getWidth();
            localLayoutParams.height = getHeight();
            int location[] = new int[2];
            getLocationOnScreen(location);
            localLayoutParams.x = location[0];
            localLayoutParams.y = location[1];
            mBrightnessDialog.getWindow().setAttributes(localLayoutParams);
        }
        if (!mBrightnessDialog.isShowing()) {
            mBrightnessDialog.show();
        }
        if (mBrightnessDialogTv != null)
            mBrightnessDialogTv.setText((int) (percent * 100) + "%");
    }

    private void dismissProgressDialog() {
        if (mProgressDialog != null) {
            mProgressDialog.dismiss();
            mProgressDialog = null;
        }
    }

    private void dismissVolumeDialog() {
        if (mVolumeDialog != null) {
            mVolumeDialog.dismiss();
            mVolumeDialog = null;
        }
    }

    private void dismissBrightnessDialog() {
        if (mBrightnessDialog != null) {
            mBrightnessDialog.dismiss();
            mBrightnessDialog = null;
        }
    }

    public void onStart() {
        resetProgressAndTime();
    }

    public void onPrepared() {
        resetProgressAndTime();
        startProgressTimer();
    }

    public void onCompletion() {
        updateStartPlay();
        cancelProgressTimer();
        if (seekBar != null) {
            seekBar.setProgress(100);
        }
        if (tvCurrent != null && tvTotal != null) {
            tvCurrent.setText(tvTotal.getText());
        }
    }

    public void onError() {
        updateStartPlay();
    }

    //进度条 定时
    private void startProgressTimer() {
        cancelProgressTimer();
        mPostProgress = true;
        postDelayed(progressTask, 300);
    }

    private void cancelProgressTimer() {
        mPostProgress = false;
        removeCallbacks(progressTask);
    }

    private void setTextAndProgress() {
        int position = getCurrentPositionWhenPlaying();
        int duration = getDuration();
        int progress = position * 100 / (duration == 0 ? 1 : duration);
        setProgressAndTime(progress, position, duration);
    }

    private void setProgressAndTime(int progress, int currentTime, int totalTime) {
        if (seekBar == null || tvTotal == null || tvCurrent == null) {
            return;
        }
        if (mHadSeekTouch) {
            return;
        }

        if (progress != 0) {
            seekBar.setProgress(progress);
        }

        tvTotal.setText(CommonUtil.stringForTime(totalTime));
        if (currentTime > 0) {
            tvCurrent.setText(CommonUtil.stringForTime(currentTime));
        }
    }

    /**
     * 初始化进度
     */
    private void resetProgressAndTime() {
        if (seekBar == null || tvTotal == null || tvCurrent == null) {
            return;
        }
        seekBar.setProgress(0);
        tvCurrent.setText(CommonUtil.stringForTime(0));
        tvTotal.setText(CommonUtil.stringForTime(0));
    }

    /**
     * 获取当前播放进度
     */
    private int getCurrentPositionWhenPlaying() {
        int position = 0;
        if (mPlayer.isPlaying() || mPlayer.isPaused()) {
            try {
                position = mPlayer.getCurrentPosition();
            } catch (Exception e) {
                e.printStackTrace();
                return position;
            }
        }
        return position;
    }

    /**
     * 获取当前总时长
     */
    private int getDuration() {
        int duration = 0;
        try {
            duration = mPlayer.getDuration();
        } catch (Exception e) {
            e.printStackTrace();
            return duration;
        }
        return duration;
    }

    /**
     * 控制栏显示 定时
     */
    private void startDismissControlViewTimer() {
        if (mPlayer == null) {
            return;
        }
        cancelDismissControlViewTimer();
        mPostDismiss = true;
        postDelayed(dismissControlTask, mDismissControlTime);
    }

    private void cancelDismissControlViewTimer() {
        if (mPlayer == null) {
            return;
        }
        mPostDismiss = false;
        removeCallbacks(dismissControlTask);
    }

    Runnable progressTask = new Runnable() {
        @Override
        public void run() {
            if (mPlayer != null && (mPlayer.isPlaying() || mPlayer.isPaused())) {
                setTextAndProgress();
            }
            if (mPostProgress) {
                postDelayed(this, 1000);
            }
        }
    };

    Runnable dismissControlTask = new Runnable() {
        @Override
        public void run() {
            if (mPlayer != null && !mPlayer.isError()) {
                hideAllWidget();
                if (mPostDismiss) {
                    postDelayed(this, mDismissControlTime);
                }
            }
        }
    };

    private void hideAllWidget() {
        setViewShowState(llBottom, INVISIBLE);
        setViewShowState(btnStart, INVISIBLE);
        mShowing = false;
    }

    private void showAllWidget() {
        setViewShowState(llBottom, VISIBLE);
        setViewShowState(btnStart, VISIBLE);
        mShowing = true;
        updateStartPlay();
    }

    private void setViewShowState(View view, int visibility) {
        if (view != null) {
            view.setVisibility(visibility);
        }
    }


    @Override
    public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
        if (fromUser) {
            int duration = getDuration();
            if (tvCurrent != null) {
                tvCurrent.setText(CommonUtil.stringForTime(progress * duration / 100));
            }
        }
    }

    @Override
    public void onStartTrackingTouch(SeekBar seekBar) {
        mHadSeekTouch = true;
    }

    @Override
    public void onStopTrackingTouch(SeekBar seekBar) {
        if (mPlayer != null) {
            try {
                int time = seekBar.getProgress() * getDuration() / 100;
                mPlayer.seekTo(time);
            } catch (Exception ignored) {
            }
        }
        mHadSeekTouch = false;
    }

    public ImageView getFullscreenButton() {
        return ivFullscreen;
    }

    /**
     * 开始全屏
     */
    private void startWindowFullscreen() {
        CommonUtil.hideSupportActionBar(getContext());
        final ViewGroup vp = getViewGroup();

        ViewGroup viewGroup = (ViewGroup) getParent();
        mRawViewGroup = viewGroup;
        viewGroup.removeAllViews();

        FrameLayout.LayoutParams lpParent = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        FrameLayout frameLayout = new FrameLayout(getContext());
        frameLayout.setId(FULLSCREEN_ID);
        frameLayout.setBackgroundColor(Color.BLACK);

        FrameLayout.LayoutParams lp = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        lp.setMargins(0, 0, 0, 0);
        lp.gravity = Gravity.CENTER;
        frameLayout.addView(mPlayer, lp);
        frameLayout.addView(this, lp);
        vp.addView(frameLayout, lpParent);
        setLayoutParams(lp);
        mPlayer.setVisibility(INVISIBLE);
        frameLayout.setVisibility(INVISIBLE);

        //旋转屏幕
        if (mIsLand == LAND_TYPE_NULL) {
            mActivity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
            if (getFullscreenButton() != null) {
                getFullscreenButton().setImageResource(getShrinkImageRes());
            }
            mIsLand = LAND_TYPE_NORMAL;
        }

        startProgressTimer();
        startDismissControlViewTimer();

        mPlayer.setVisibility(VISIBLE);
        frameLayout.setVisibility(VISIBLE);

        mIfCurrentIsFullscreen = true;
    }

    /**
     * 取消全屏
     */
    private void clearFullscreenLayout() {
        mIfCurrentIsFullscreen = false;
        backToProtVideo();
        ViewGroup vp = getViewGroup();
        ViewGroup oldF = vp.findViewById(FULLSCREEN_ID);
        MuduVideoView view;
        if (oldF != null) {
            view = (MuduVideoView) oldF.getChildAt(0);
            if (view != null) {
                resolveNormalVideoShow(oldF, vp, view);
            }
        } else {
            resolveNormalVideoShow(null, vp, null);
        }
    }

    @SuppressLint("SourceLockedOrientationActivity")
    private void backToProtVideo() {
        if (mIsLand > LAND_TYPE_NULL) {
            mActivity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
            if (getFullscreenButton() != null) {
                getFullscreenButton().setImageResource(getEnlargeImageRes());
            }
            mIsLand = LAND_TYPE_NULL;
        }
    }

    private void resolveNormalVideoShow(ViewGroup oldF, ViewGroup vp, MuduVideoView view) {
        if (oldF != null && oldF.getParent() != null) {
            oldF.removeAllViews();
            vp.removeView(oldF);
        }

        FrameLayout.LayoutParams lpVideo = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        FrameLayout.LayoutParams lpController = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        mRawViewGroup.addView(view, lpVideo);
        mRawViewGroup.addView(this, lpController);

        startProgressTimer();
        startDismissControlViewTimer();
        CommonUtil.showSupportActionBar(getContext());
    }

    /**
     * 退出全屏，主要用于返回键
     *
     * @return 返回是否全屏
     */
    public boolean backFromWindowFull() {
        boolean backFrom = false;
        ViewGroup vp = getViewGroup();
        ViewGroup oldF = vp.findViewById(FULLSCREEN_ID);
        if (oldF != null) {
            backFrom = true;
            clearFullscreenLayout();
        }
        return backFrom;
    }

    private int getShrinkImageRes() {
        return R.mipmap.video_shrink;
    }

    private int getEnlargeImageRes() {
        return R.mipmap.video_enlarge;
    }

    private ViewGroup getViewGroup() {
        return (CommonUtil.scanForActivity(getContext())).findViewById(Window.ID_ANDROID_CONTENT);
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        cancelProgressTimer();
        cancelDismissControlViewTimer();
    }
}
