package mudu.tv.mudutv_android_player;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.mudu.yaguplayer.utils.MuduVideoType;
import com.mudu.yaguplayer.video.widget.media.MuduVideoView;

import java.util.ArrayList;

import tv.danmaku.ijk.media.muduplayer.IMediaPlayer;
import tv.mudu.commentlib.Commenter;
import tv.mudu.commentlib.entity.MessageEntity;

public class MainActivity extends AppCompatActivity {

    private final static String TAG = "PlayActivity";

    private CustomController mLayoutController;
    private MuduVideoView mVideoView;

    private Button btnReplay;
    private Button btnShot;
    private Button btnOrientation;
    private Button btnSpeed;
    private Button btnRatio;
    private Button btnRadius;

    private String mVideoPath = "https://mov.bn.netease.com/open-movie/nos/mp4/2015/08/27/SB13F5AGJ_sd.mp4";

    private final float[] speeds = {0.5F, 1.0F, 1.5F, 2.0F};
    private final int[] ratios = {MuduVideoType.AR_ASPECT_FIT_PARENT, MuduVideoType.AR_4_3_FIT_PARENT, MuduVideoType.AR_16_9_FIT_PARENT, MuduVideoType.AR_ASPECT_FILL_PARENT, MuduVideoType.AR_MATCH_PARENT};
    private int speedIndex = 1;
    private int ratioIndex = 0;

    public static final String INTENT_PATH = "intent_path";

    private Commenter commenter;
    private boolean commentClosed = false;
    private Handler handler;

    private ListView listview;
    private MyAdapter adapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        String intentPath = getIntent().getStringExtra(INTENT_PATH);
        if (!TextUtils.isEmpty(intentPath)) {
            mVideoPath = intentPath;
        }
        mVideoView = findViewById(R.id.video_view);
        initControlView();
        initVideoListener();

        initListView();
        initHandler();
        initCommenter();
        initCommenterBtn();
        start();
    }

    private void initControlView() {
        mLayoutController = findViewById(R.id.layout_controller);
        mVideoView.setMediaController(mLayoutController);
        btnReplay = findViewById(R.id.btn_replay);
        btnShot = findViewById(R.id.btn_shot);
        btnOrientation = findViewById(R.id.btn_orientation);
        btnSpeed = findViewById(R.id.btn_speed);
        btnRatio = findViewById(R.id.btn_ratio);
        btnRadius = findViewById(R.id.btn_radius);
        btnReplay.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                start();
            }
        });

        btnShot.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mVideoView.getScreenImage(new MuduVideoView.IScreenImageCallBack() {
                    @Override
                    public void onResult(Bitmap bitmap) {
                        Toast.makeText(MainActivity.this, "已获取到截图", Toast.LENGTH_SHORT).show();
                    }
                });
            }
        });

        btnRatio.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (ratioIndex == 4) {
                    ratioIndex = 0;
                } else {
                    ratioIndex++;
                }

                mVideoView.setAspectRatio(ratios[ratioIndex]);
                if (ratioIndex == 0) {
                    btnRatio.setText("默认比例");
                } else if (ratioIndex == 1) {
                    btnRatio.setText("4:3");
                } else if (ratioIndex == 2) {
                    btnRatio.setText("16:9");
                } else if (ratioIndex == 3) {
                    btnRatio.setText("裁剪全屏");
                } else if (ratioIndex == 4) {
                    btnRatio.setText("拉伸全屏");
                }
            }
        });

        btnOrientation.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT) {
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
                } else {
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
                }
            }
        });

        btnSpeed.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (speedIndex == 3) {
                    speedIndex = 0;
                } else {
                    speedIndex++;
                }

                mVideoView.setSpeed(speeds[speedIndex]);
                btnSpeed.setText("播放速度 " + speeds[speedIndex]);
            }
        });

        btnRadius.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mVideoView.setVideoRadius(200);
            }
        });

        findViewById(R.id.start).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                String url = ((EditText)findViewById(R.id.inputUrl)).getText().toString();
                mVideoView.setUrl(url);
                mVideoView.start();
            }
        });
        findViewById(R.id.stop).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mVideoView.release();
            }
        });
        ((EditText)findViewById(R.id.inputUrl)).setText(mVideoPath);
    }

    private void initVideoListener() {
        mVideoView.setOnPreparedListener(new IMediaPlayer.OnPreparedListener() {
            @Override
            public void onPrepared(IMediaPlayer mp) {
                mLayoutController.onPrepared();
                Log.e("mVideoView", "OnPreparedListener onPrepared");
                Log.e(TAG, "mCurrentPosition = " + mp.getCurrentPosition());
            }
        });

        mVideoView.setOnErrorListener(new IMediaPlayer.OnErrorListener() {
            @Override
            public boolean onError(IMediaPlayer mp, int what, int extra) {
                mLayoutController.onError();
                Log.e("mVideoView", what + "  " + extra);
                Log.e(TAG, "currentTime = " + mp.getCurrentPosition());
                return false;
            }
        });
        mVideoView.setOnCompletionListener(new IMediaPlayer.OnCompletionListener() {
            @Override
            public void onCompletion(IMediaPlayer mp) {
                Log.d(TAG, "onComplete");
                mLayoutController.onCompletion();
            }
        });
        mVideoView.setOnSeekCompleteListener(new IMediaPlayer.OnSeekCompleteListener() {
            @Override
            public void onSeekComplete(IMediaPlayer mp) {
                Log.e(TAG, "seek position = " + mVideoView.getCurrentPosition());
                Log.e(TAG, "seek total duration = " + mVideoView.getDuration());
                Log.e(TAG, "onSeek Complete");
            }
        });
        mVideoView.setOnInfoListener(new IMediaPlayer.OnInfoListener() {
            @Override
            public boolean onInfo(IMediaPlayer mp, int arg1, int arg2) {
                Log.e(TAG, "mVideoView info " + arg1 + "  " + arg2);

                switch (arg1) {
                    case IMediaPlayer.MEDIA_INFO_VIDEO_TRACK_LAGGING:
                        Log.e(TAG, "MEDIA_INFO_VIDEO_TRACK_LAGGING:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_VIDEO_RENDERING_START:
                        Log.e(TAG, "MEDIA_INFO_VIDEO_RENDERING_START  ");
                        break;
                    case IMediaPlayer.MEDIA_INFO_BUFFERING_START:
                        Log.e(TAG, "MEDIA_INFO_BUFFERING_START:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_BUFFERING_END:
                        Log.e(TAG, "MEDIA_INFO_BUFFERING_END:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_NETWORK_BANDWIDTH:
                        Log.e(TAG, "MEDIA_INFO_NETWORK_BANDWIDTH: arg1" + arg1 + "  arg2: " + arg2);
                        break;
                    case IMediaPlayer.MEDIA_INFO_BAD_INTERLEAVING:
                        Log.e(TAG, "MEDIA_INFO_BAD_INTERLEAVING:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_NOT_SEEKABLE:
                        Log.e(TAG, "MEDIA_INFO_NOT_SEEKABLE:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_METADATA_UPDATE:
                        Log.e(TAG, "MEDIA_INFO_METADATA_UPDATE:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_UNSUPPORTED_SUBTITLE:
                        Log.e(TAG, "MEDIA_INFO_UNSUPPORTED_SUBTITLE:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_SUBTITLE_TIMED_OUT:
                        Log.e(TAG, "MEDIA_INFO_SUBTITLE_TIMED_OUT:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_VIDEO_ROTATION_CHANGED:
                        Log.e(TAG, "MEDIA_INFO_VIDEO_ROTATION_CHANGED: " + arg2);
                        break;
                    case IMediaPlayer.MEDIA_INFO_NETWORK_SPEED:
                        break;
                    case IMediaPlayer.MEDIA_INFO_PKTSIZE:
                        break;
                    case IMediaPlayer.MEDIA_INFO_AUDIO_RENDERING_START:
                        Log.e(TAG, "MEDIA_INFO_AUDIO_RENDERING_START:");
                        break;
                    case IMediaPlayer.MEDIA_INFO_PLAY_DISCONTINUOUS:
                        Log.e(TAG, " getmessage MEDIA_INFO_PLAY_DISCONTINUOUS time = ");
                        break;
                    case IMediaPlayer.MEDIA_INFO_DOMAINANALYTIC_TIME:
                        int time1 = (int) arg2;
                        Log.e(TAG, "Test MEDIA_INFO_DOMAINANALYTIC_TIME = " + time1);
                        break;
                    case IMediaPlayer.MEDIA_INFO_CONNECT_TIME:
                        int time2 = (int) arg2;
                        Log.e(TAG, "Test MEDIA_INFO_CONNECT_TIME = " + time2);
                        break;
                    case IMediaPlayer.MEDIA_INFO_AUDIO_SEEK_RENDERING_START:
                        Log.e(TAG, "Test MEDIA_INFO_AUDIO_SEEK_RENDERING_START");
                        break;
                    case IMediaPlayer.MEDIA_INFO_AUDIO_FPS:
                        Log.e(TAG, "Test MEDIA_INFO_AUDIO_FPS = " + arg2);
                        break;
                }
                return false;
            }
        });
    }

    private void initHandler() {
        handler = new Handler(Looper.myLooper()) {
            @Override
            public void handleMessage(Message msg) {
                String msgstr = msg.getData().getString("body");
                adapter.arr.add(msgstr);
                adapter.notifyDataSetChanged();
                listview.setSelection(listview.getBottom());
            }
        };
    }

    private void initListView() {
        listview = findViewById(R.id.listview);
        adapter = new MyAdapter(this);
        listview.setAdapter(adapter);
    }

    private void start() {
        if (!"".equals(mVideoPath)) {
            mVideoView.setUrl(mVideoPath);
            mVideoView.setPTSInterval(500);
            mVideoView.start();
            mLayoutController.onStart();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        mVideoView.resume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        Log.e(TAG, "MainActivity onPause");
        mVideoView.pause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.e(TAG, "MainActivity onDestroy");
        mVideoView.release();
    }

    @Override
    public void onBackPressed() {
        if (mLayoutController.backFromWindowFull()) {
            return;
        }
        super.onBackPressed();
    }

    private void initCommenter() {
        // 创建评论管理
        commenter = new Commenter("mz1ye3rl", this);
        // 设置头像
        commenter.login("评论者", "http://mudu.tv/assets/img/icon.png", new Commenter.Callback() {
            @Override
            public void onSuccess(String s) {
                String userInfo = commenter.getUserInfo()==null?"":commenter.getUserInfo().toString();
                showMsg(userInfo);
            }

            @Override
            public void onError(Exception e) {
                //登录失败
            }
        });
        // 进行连接频道
        commenter.subscribe(new Commenter.IConnectCallback() {

            @Override
            public void handle(int connectCode, String error) {

                if (connectCode == Commenter.ConnectCode_Success) {
                    showMsg("连接成功");
                } else if (connectCode == Commenter.ConnectCode_Error) {
                    showMsg(error);

                } else if (connectCode == Commenter.ConnectCode_Close) {
                    if(!commentClosed){
                        //考虑断网情况，请自行制定策略去重连
                        new Handler().postDelayed(new Runnable() {
                            @Override
                            public void run() {
                                commenter.reconnect();
                            }
                        },  1000);
                    }
                }

            }
        }, new Commenter.IMessageCallback(){
            @Override
            public void handle(String msg, String type) {
                //获取消息
                showMsg("type:" + type + "  msg:" + msg);
            }

            @Override
            public void message(MessageEntity entity, String s) {
                showMsg(entity.toString());
            }
        });
    }

    private void initCommenterBtn() {
        EditText editText = (EditText) findViewById(R.id.editText);
        findViewById(R.id.add).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View arg0) {
                String editStr = editText.getText().toString();
                if ("".equals(editStr)) {
                    showMsg("please input some message  first !");
                    return;
                }

                commenter.pubilsh(editText.getText().toString(), new Commenter.Callback() {

                    @Override
                    public void onSuccess(String s) {
                        showMsg("发送成功");
                    }

                    @Override
                    public void onError(Exception e) {
                        //发送失败
                    }
                });
                editText.setText("");
            }
        });

        findViewById(R.id.close).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                commentClosed = true;
                commenter.unsubscribe();
                commenter.close();

                finish();
            }
        });
    }

    public void showMsg(String body) {
        Message message = new Message();
        Bundle bundle = new Bundle();
        bundle.putString("body", body);
        message.setData(bundle);
        handler.sendMessage(message);
    }

    private class MyAdapter extends BaseAdapter {

        private Context context;
        private LayoutInflater inflater;
        public ArrayList<String> arr;

        public MyAdapter(Context context) {
            super();
            this.context = context;
            inflater = LayoutInflater.from(context);
            arr = new ArrayList<String>();
        }

        @Override
        public int getCount() {
            // TODO Auto-generated method stub
            return arr.size();
        }

        @Override
        public Object getItem(int arg0) {
            // TODO Auto-generated method stub
            return arg0;
        }

        @Override
        public long getItemId(int arg0) {
            // TODO Auto-generated method stub
            return arg0;
        }

        @Override
        public View getView(final int position, View view, ViewGroup arg2) {
            // TODO Auto-generated method stub
            if (view == null) {
                view = inflater.inflate(R.layout.item, null);
            }
            final TextView text = (TextView) view.findViewById(R.id.textView);
            String str = arr.get(position);
            text.setText(str);
            text.setTextColor(Color.RED);
            return view;
        }
    }
}